#    Copyright 2014 Objectif Libre
#    Copyright 2015 DotHill Systems
#    Copyright 2016-19 Seagate Technology or one of its affiliates
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
"""Unit tests for OpenStack Cinder Seagate driver."""

from unittest import mock

from lxml import etree
import requests

from cinder import exception
from cinder.objects import fields
from cinder.tests.unit import test
import cinder.volume.drivers.stx.client
import cinder.volume.drivers.stx.common
import cinder.volume.drivers.stx.exception as stx_exception
import cinder.volume.drivers.stx.fc
import cinder.volume.drivers.stx.iscsi
from cinder.zonemanager import utils as fczm_utils


STXClient = cinder.volume.drivers.stx.client.STXClient
STXCommon = cinder.volume.drivers.stx.common.STXCommon
STXFCDriver = cinder.volume.drivers.stx.fc.STXFCDriver
STXISCSIDriver = cinder.volume.drivers.stx.iscsi.STXISCSIDriver

session_key = '12a1626754554a21d85040760c81b'
resp_login = '''<RESPONSE><OBJECT basetype="status" name="status" oid="1">
             <PROPERTY name="response-type">success</PROPERTY>
             <PROPERTY name="response-type-numeric">0</PROPERTY>
             <PROPERTY name="response">12a1626754554a21d85040760c81b</PROPERTY>
             <PROPERTY name="return-code">1</PROPERTY></OBJECT></RESPONSE>'''

resp_fw_ti = '''<RESPONSE><PROPERTY name="sc-fw">T252R07</PROPERTY>
                       <PROPERTY name="return-code">0</PROPERTY></RESPONSE>'''

resp_fw = '''<RESPONSE><PROPERTY name="sc-fw">GLS220R001</PROPERTY>
                       <PROPERTY name="return-code">0</PROPERTY></RESPONSE>'''

resp_fw_nomatch = '''<RESPONSE><PROPERTY name="sc-fw">Z</PROPERTY>
                       <PROPERTY name="return-code">0</PROPERTY></RESPONSE>'''

resp_system = '''<RESPONSE>
             <PROPERTY name="midplane-serial-number">00C0FFEEEEEE</PROPERTY>
             <PROPERTY name="return-code">0</PROPERTY>
                 </RESPONSE>'''

resp_badlogin = '''<RESPONSE><OBJECT basetype="status" name="status" oid="1">
             <PROPERTY name="response-type">error</PROPERTY>
             <PROPERTY name="response-type-numeric">1</PROPERTY>
             <PROPERTY name="response">Authentication failure</PROPERTY>
             <PROPERTY name="return-code">1</PROPERTY></OBJECT></RESPONSE>'''
response_ok = '''<RESPONSE><OBJECT basetype="status" name="status" oid="1">
              <PROPERTY name="response">some data</PROPERTY>
              <PROPERTY name="return-code">0</PROPERTY>
              </OBJECT></RESPONSE>'''
response_not_ok = '''<RESPONSE><OBJECT basetype="status" name="status" oid="1">
                  <PROPERTY name="response">Error Message</PROPERTY>
                  <PROPERTY name="return-code">1</PROPERTY>
                  </OBJECT></RESPONSE>'''
response_stats_linear = '''<RESPONSE><OBJECT basetype="virtual-disks">
                    <PROPERTY name="size-numeric">3863830528</PROPERTY>
                    <PROPERTY name="freespace-numeric">3863830528</PROPERTY>
                    </OBJECT></RESPONSE>'''
response_stats_virtual = '''<RESPONSE><OBJECT basetype="pools">
                <PROPERTY name="total-size-numeric">3863830528</PROPERTY>
                <PROPERTY name="total-avail-numeric">3863830528</PROPERTY>
                </OBJECT></RESPONSE>'''
response_no_lun = '''<RESPONSE></RESPONSE>'''
response_lun = '''<RESPONSE><OBJECT basetype="host-view-mappings">
               <PROPERTY name="lun">1</PROPERTY></OBJECT>
               <OBJECT basetype="host-view-mappings">
               <PROPERTY name="lun">4</PROPERTY></OBJECT></RESPONSE>'''
response_ports = '''<RESPONSE>
                 <OBJECT basetype="port">
                 <PROPERTY name="port-type">FC</PROPERTY>
                 <PROPERTY name="target-id">id1</PROPERTY>
                 <PROPERTY name="status">Disconnected</PROPERTY></OBJECT>
                 <OBJECT basetype="port">
                 <PROPERTY name="port-type">FC</PROPERTY>
                 <PROPERTY name="target-id">id2</PROPERTY>
                 <PROPERTY name="status">Up</PROPERTY></OBJECT>
                 <OBJECT basetype="port">
                 <PROPERTY name="port-type">iSCSI</PROPERTY>
                 <PROPERTY name="target-id">id3</PROPERTY>
                 <PROPERTY name="%(ip)s" >10.0.0.10</PROPERTY>
                 <PROPERTY name="status">Disconnected</PROPERTY></OBJECT>
                 <OBJECT basetype="port">
                 <PROPERTY name="port-type">iSCSI</PROPERTY>
                 <PROPERTY name="target-id">id4</PROPERTY>
                 <PROPERTY name="%(ip)s" >10.0.0.11</PROPERTY>
                 <PROPERTY name="status">Up</PROPERTY></OBJECT>
                 <OBJECT basetype="port">
                 <PROPERTY name="port-type">iSCSI</PROPERTY>
                 <PROPERTY name="target-id">id5</PROPERTY>
                 <PROPERTY name="%(ip)s" >10.0.0.12</PROPERTY>
                 <PROPERTY name="status">Up</PROPERTY></OBJECT>
                 </RESPONSE>'''

# mccli -x array show-volumes | egrep \
# '(RESPONSE|OBJECT|volume-name|volume-type|serial-number|wwn| \
#   storage-pool-name|size-numeric|volume-parent)'
response_vols = '''
<RESPONSE VERSION="L100" REQUEST="show volumes">
  <OBJECT basetype="volumes" name="volume" oid="1" format="rows">
    <PROPERTY name="storage-pool-name" type="string">A</PROPERTY>
    <PROPERTY name="volume-name" type="string">bar</PROPERTY>
    <PROPERTY name="size-numeric" type="uint64">2097152</PROPERTY>
    <PROPERTY name="total-size-numeric" type="uint64">2097152</PROPERTY>
    <PROPERTY name="allocated-size-numeric" type="uint64">0</PROPERTY>
    <PROPERTY name="serial-number" key="true"
              type="string">00c0ff53a30500000323416101000000</PROPERTY>
    <PROPERTY name="read-ahead-size-numeric" type="uint32">-1</PROPERTY>
    <PROPERTY name="volume-type" type="string">base</PROPERTY>
    <PROPERTY name="volume-type-numeric" type="uint32">15</PROPERTY>
    <PROPERTY name="volume-parent" type="string"></PROPERTY>
    <PROPERTY name="wwn"
              type="string">600C0FF00053A3050323416101000000</PROPERTY>
  </OBJECT>
  <OBJECT basetype="volumes" name="volume" oid="2" format="rows">
    <PROPERTY name="storage-pool-name" type="string">A</PROPERTY>
    <PROPERTY name="volume-name" type="string">foo</PROPERTY>
    <PROPERTY name="size-numeric" type="uint64">8388608</PROPERTY>
    <PROPERTY name="total-size-numeric" type="uint64">8388608</PROPERTY>
    <PROPERTY name="allocated-size-numeric" type="uint64">0</PROPERTY>
    <PROPERTY name="serial-number" key="true"
              type="string">00c0ff53a3050000df513e6101000000</PROPERTY>
    <PROPERTY name="read-ahead-size-numeric" type="uint32">-1</PROPERTY>
    <PROPERTY name="volume-type" type="string">base</PROPERTY>
    <PROPERTY name="volume-type-numeric" type="uint32">15</PROPERTY>
    <PROPERTY name="volume-parent" type="string"></PROPERTY>
    <PROPERTY name="wwn"
              type="string">600C0FF00053A305DF513E6101000000</PROPERTY>
  </OBJECT>
  <OBJECT basetype="volumes" name="volume" oid="5" format="rows">
    <PROPERTY name="storage-pool-name" type="string">A</PROPERTY>
    <PROPERTY name="volume-name" type="string">snap</PROPERTY>
    <PROPERTY name="size-numeric" type="uint64">2097152</PROPERTY>
    <PROPERTY name="total-size-numeric" type="uint64">2097152</PROPERTY>
    <PROPERTY name="allocated-size-numeric" type="uint64">0</PROPERTY>
    <PROPERTY name="serial-number" key="true"
              type="string">00c0ff53a3050000fbc5416101000000</PROPERTY>
    <PROPERTY name="read-ahead-size-numeric" type="uint32">-1</PROPERTY>
    <PROPERTY name="volume-type" type="string">snapshot</PROPERTY>
    <PROPERTY name="volume-type-numeric" type="uint32">13</PROPERTY>
    <PROPERTY name="volume-parent"
              type="string">00c0ff53a30500000323416101000000</PROPERTY>
    <PROPERTY name="wwn"
              type="string">600C0FF00053A305FBC5416101000000</PROPERTY>
  </OBJECT>
  <OBJECT basetype="volumes" name="volume" oid="6" format="rows">
    <PROPERTY name="storage-pool-name" type="string">A</PROPERTY>
    <PROPERTY name="volume-name" type="string">vqoINx4UbS-Cno3gDq1V</PROPERTY>
    <PROPERTY name="size-numeric" type="uint64">2097152</PROPERTY>
    <PROPERTY name="total-size-numeric" type="uint64">2097152</PROPERTY>
    <PROPERTY name="allocated-size-numeric" type="uint64">0</PROPERTY>
    <PROPERTY name="serial-number" key="true"
              type="string">00c0ff53a305000024c6416101000000</PROPERTY>
    <PROPERTY name="read-ahead-size-numeric" type="uint32">-1</PROPERTY>
    <PROPERTY name="volume-type" type="string">base</PROPERTY>
    <PROPERTY name="volume-type-numeric" type="uint32">15</PROPERTY>
    <PROPERTY name="volume-parent" type="string"></PROPERTY>
    <PROPERTY name="wwn"
              type="string">600C0FF00053A30524C6416101000000</PROPERTY>
  </OBJECT>
  <OBJECT basetype="status" name="status" oid="7">
  </OBJECT>
</RESPONSE>
'''

response_maps = '''
<RESPONSE VERSION="L100" REQUEST="show maps">
  <OBJECT basetype="volume-view" name="volume-view" oid="1" format="labeled">
    <PROPERTY name="volume-serial" key="true"
              type="string">00c0ff53a30500000323416101000000</PROPERTY>
    <PROPERTY name="volume-name" type="string">bar</PROPERTY>
  </OBJECT>
</RESPONSE>
'''

# The two XML samples above will produce the following result from
# get_manageable_volumes():
#
# [{'cinder_id': None,
#   'extra_info': None,
#   'reason_not_safe': 'volume in use',
#   'reference': {'source-name': 'bar'},
#   'safe_to_manage': False,
#   'size': 1},
#  {'cinder_id': 'aa820dc7-851b-4be0-a7a3-7803ab555495',
#   'extra_info': None,
#   'reason_not_safe': 'already managed',
#   'reference': {'source-name': 'vqoINx4UbS-Cno3gDq1V'},
#   'safe_to_manage': False,
#   'size': 1},
#  {'cinder_id': None,
#   'extra_info': None,
#   'reason_not_safe': None,
#   'reference': {'source-name': 'foo'},
#   'safe_to_manage': True,
#   'size': 4}]

response_ports_linear = response_ports % {'ip': 'primary-ip-address'}
response_ports_virtual = response_ports % {'ip': 'ip-address'}


invalid_xml = '''<RESPONSE></RESPONSE>'''
malformed_xml = '''<RESPONSE>'''
fake_xml = '''<fakexml></fakexml>'''

stats_low_space = {'free_capacity_gb': 10, 'total_capacity_gb': 100}
stats_large_space = {'free_capacity_gb': 90, 'total_capacity_gb': 100}

vol_id = 'fceec30e-98bc-4ce5-85ff-d7309cc17cc2'
test_volume = {'id': vol_id, 'name_id': None,
               'display_name': 'test volume', 'name': 'volume', 'size': 10}
test_retype_volume = {'attach_status': fields.VolumeAttachStatus.DETACHED,
                      'id': vol_id, 'name_id': None,
                      'display_name': 'test volume', 'name': 'volume',
                      'size': 10}
test_host = {'capabilities': {'location_info':
                              'SeagateVolumeDriver:xxxxx:dg02:A'}}
test_snap = {'id': 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa',
             'volume': {'name_id': None},
             'volume_id': vol_id, 'display_name': 'test volume',
             'name': 'volume', 'volume_size': 10}
encoded_volid = 'v_O7DDpi8TOWF_9cwnMF'
encoded_snapid = 's_O7DDpi8TOWF_9cwnMF'
dest_volume = {'id': 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa',
               'source_volid': vol_id,
               'display_name': 'test volume', 'name': 'volume', 'size': 10}
dest_volume_larger = {'id': 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa',
                      'name_id': None,
                      'source_volid': vol_id,
                      'display_name': 'test volume',
                      'name': 'volume', 'size': 20}
attached_volume = {'id': vol_id,
                   'display_name': 'test volume', 'name': 'volume',
                   'size': 10, 'status': 'in-use',
                   'attach_status': fields.VolumeAttachStatus.ATTACHED}
attaching_volume = {'id': vol_id,
                    'display_name': 'test volume', 'name': 'volume',
                    'size': 10, 'status': 'attaching',
                    'attach_status': fields.VolumeAttachStatus.ATTACHED}
detached_volume = {'id': vol_id, 'name_id': None,
                   'display_name': 'test volume', 'name': 'volume',
                   'size': 10, 'status': 'available',
                   'attach_status': 'detached'}

connector = {'ip': '10.0.0.2',
             'initiator': 'iqn.1993-08.org.debian:01:222',
             'wwpns': ["111111111111111", "111111111111112"],
             'wwnns': ["211111111111111", "211111111111112"],
             'host': 'fakehost'}
invalid_connector = {'ip': '10.0.0.2',
                     'initiator': '',
                     'wwpns': [],
                     'wwnns': [],
                     'host': 'fakehost'}


class TestSeagateClient(test.TestCase):
    def setUp(self):
        super(TestSeagateClient, self).setUp()
        self.login = 'manage'
        self.passwd = '!manage'
        self.ip = '10.0.0.1'
        self.protocol = 'http'
        self.ssl_verify = False
        self.client = STXClient(self.ip, self.login, self.passwd,
                                self.protocol, self.ssl_verify)

    @mock.patch('requests.get')
    def test_login(self, mock_requests_get):
        m = mock.Mock()
        mock_requests_get.return_value = m

        m.text.encode.side_effect = [resp_badlogin, resp_badlogin]
        self.assertRaises(stx_exception.AuthenticationError,
                          self.client.login)

        m.text.encode.side_effect = [resp_login, resp_fw_nomatch, resp_system]
        self.client.login()
        self.assertEqual('Z', self.client._fw_type)
        self.assertEqual(0, self.client._fw_rev)
        self.assertEqual(False, self.client.is_g5_fw())

        m.text.encode.side_effect = [resp_login, resp_fw, resp_system]
        self.client.login()
        self.assertEqual(session_key, self.client._session_key)

    def test_build_request_url(self):
        url = self.client._build_request_url('/path')
        self.assertEqual('http://10.0.0.1/api/path', url)
        url = self.client._build_request_url('/path', arg1='val1')
        self.assertEqual('http://10.0.0.1/api/path/arg1/val1', url)
        url = self.client._build_request_url('/path', arg_1='val1')
        self.assertEqual('http://10.0.0.1/api/path/arg-1/val1', url)
        url = self.client._build_request_url('/path', 'arg1')
        self.assertEqual('http://10.0.0.1/api/path/arg1', url)
        url = self.client._build_request_url('/path', 'arg1', arg2='val2')
        self.assertEqual('http://10.0.0.1/api/path/arg2/val2/arg1', url)
        url = self.client._build_request_url('/path', 'arg1', 'arg3',
                                             arg2='val2')
        self.assertEqual('http://10.0.0.1/api/path/arg2/val2/arg1/arg3', url)

    @mock.patch('requests.get')
    def test_request(self, mock_requests_get):
        self.client._session_key = session_key

        m = mock.Mock()
        m.text.encode.side_effect = [response_ok, malformed_xml,
                                     requests.exceptions.
                                     RequestException("error")]
        mock_requests_get.return_value = m
        ret = self.client._api_request('/path')
        self.assertTrue(type(ret) == etree._Element)
        self.assertRaises(stx_exception.ConnectionError,
                          self.client._api_request,
                          '/path')
        self.assertRaises(stx_exception.ConnectionError,
                          self.client._api_request,
                          '/path')

    def test_assert_response_ok(self):
        ok_tree = etree.XML(response_ok)
        not_ok_tree = etree.XML(response_not_ok)
        invalid_tree = etree.XML(invalid_xml)
        ret = self.client._assert_response_ok(ok_tree)
        self.assertIsNone(ret)
        self.assertRaises(stx_exception.RequestError,
                          self.client._assert_response_ok,
                          not_ok_tree)
        self.assertRaises(stx_exception.RequestError,
                          self.client._assert_response_ok, invalid_tree)

    @mock.patch.object(STXClient, '_request')
    def test_backend_exists(self, mock_request):
        mock_request.side_effect = [stx_exception.RequestError,
                                    fake_xml]
        self.assertFalse(self.client.backend_exists('backend_name',
                                                    'linear'))
        self.assertTrue(self.client.backend_exists('backend_name',
                                                   'linear'))

    @mock.patch.object(STXClient, '_request')
    def test_backend_stats(self, mock_request):
        stats = {'free_capacity_gb': 1843,
                 'total_capacity_gb': 1843}
        linear = etree.XML(response_stats_linear)
        virtual = etree.XML(response_stats_virtual)
        mock_request.side_effect = [linear, virtual]

        self.assertEqual(stats, self.client.backend_stats('OpenStack',
                                                          'linear'))
        self.assertEqual(stats, self.client.backend_stats('A',
                                                          'virtual'))

    @mock.patch.object(STXClient, '_request')
    def test_get_lun(self, mock_request):
        mock_request.side_effect = [etree.XML(response_no_lun),
                                    etree.XML(response_lun)]
        ret = self.client._get_first_available_lun_for_host("fakehost")
        self.assertEqual(1, ret)
        ret = self.client._get_first_available_lun_for_host("fakehost")
        self.assertEqual(2, ret)

    @mock.patch.object(STXClient, '_request')
    def test_get_ports(self, mock_request):
        mock_request.side_effect = [etree.XML(response_ports)]
        ret = self.client.get_active_target_ports()
        self.assertEqual([{'port-type': 'FC',
                           'target-id': 'id2',
                           'status': 'Up'},
                          {'port-type': 'iSCSI',
                           'target-id': 'id4',
                           'status': 'Up'},
                          {'port-type': 'iSCSI',
                           'target-id': 'id5',
                           'status': 'Up'}], ret)

    @mock.patch.object(STXClient, '_request')
    def test_get_fc_ports(self, mock_request):
        mock_request.side_effect = [etree.XML(response_ports)]
        ret = self.client.get_active_fc_target_ports()
        self.assertEqual(['id2'], ret)

    @mock.patch.object(STXClient, '_request')
    def test_get_iscsi_iqns(self, mock_request):
        mock_request.side_effect = [etree.XML(response_ports)]
        ret = self.client.get_active_iscsi_target_iqns()
        self.assertEqual(['id4', 'id5'], ret)

    @mock.patch.object(STXClient, '_request')
    def test_get_iscsi_portals(self, mock_request):
        portals = {'10.0.0.12': 'Up', '10.0.0.11': 'Up'}
        mock_request.side_effect = [etree.XML(response_ports_linear),
                                    etree.XML(response_ports_virtual)]
        ret = self.client.get_active_iscsi_target_portals()
        self.assertEqual(portals, ret)
        ret = self.client.get_active_iscsi_target_portals()
        self.assertEqual(portals, ret)

    @mock.patch.object(STXClient, '_request')
    def test_delete_snapshot(self, mock_request):
        mock_request.side_effect = [None, None]
        self.client.delete_snapshot('dummy', 'linear')
        mock_request.assert_called_with('/delete/snapshot', 'cleanup', 'dummy')
        self.client.delete_snapshot('dummy', 'paged')
        mock_request.assert_called_with('/delete/snapshot', 'dummy')

    @mock.patch.object(STXClient, '_request')
    def test_list_luns_for_host(self, mock_request):
        mock_request.side_effect = [etree.XML(response_no_lun),
                                    etree.XML(response_lun),
                                    etree.XML(response_lun)]
        self.client._fw_type = 'T'
        self.client.list_luns_for_host('dummy')
        mock_request.assert_called_with('/show/host-maps', 'dummy')
        self.client._fw_type = 'G'
        self.client.list_luns_for_host('dummy')
        mock_request.assert_called_with('/show/maps/initiator', 'dummy')
        self.client._fw_type = 'I'
        self.client.list_luns_for_host('dummy')
        mock_request.assert_called_with('/show/maps/initiator', 'dummy')


class FakeConfiguration1(object):
    seagate_pool_name = 'OpenStack'
    seagate_pool_type = 'linear'
    san_ip = '10.0.0.1'
    san_login = 'manage'
    san_password = '!manage'
    seagate_api_protocol = 'http'
    driver_use_ssl = True
    driver_ssl_cert_verify = False

    def safe_get(self, key):
        return 'fakevalue'


class FakeConfiguration2(FakeConfiguration1):
    seagate_iscsi_ips = ['10.0.0.11']
    use_chap_auth = None


class fake(dict):
    def __init__(self, *args, **kwargs):
        for d in args:
            self.update(d)
        self.update(kwargs)

    def __getattr__(self, attr):
        return self[attr]


class TestFCSeagateCommon(test.TestCase):
    def setUp(self):
        super(TestFCSeagateCommon, self).setUp()
        self.config = FakeConfiguration1()
        self.common = STXCommon(self.config)
        self.common.client_login = mock.MagicMock()
        self.common.client_logout = mock.MagicMock()
        self.common.serialNumber = "xxxxx"
        self.common.owner = "A"
        self.connector_element = "wwpns"

    @mock.patch.object(STXClient, 'get_serial_number')
    @mock.patch.object(STXClient, 'get_owner_info')
    @mock.patch.object(STXClient, 'backend_exists')
    def test_do_setup(self, mock_backend_exists,
                      mock_owner_info, mock_serial_number):
        mock_backend_exists.side_effect = [False, True]
        mock_owner_info.return_value = "A"
        mock_serial_number.return_value = "xxxxx"
        self.assertRaises(stx_exception.InvalidBackend,
                          self.common.do_setup, None)
        self.assertIsNone(self.common.do_setup(None))
        mock_backend_exists.assert_called_with(self.common.backend_name,
                                               self.common.backend_type)
        mock_owner_info.assert_called_with(self.common.backend_name,
                                           self.common.backend_type)

    def test_vol_name(self):
        self.assertEqual(encoded_volid, self.common._get_vol_name(vol_id))
        self.assertEqual(encoded_snapid, self.common._get_snap_name(vol_id))

    def test_check_flags(self):
        class FakeOptions(object):
            def __init__(self, d):
                for k, v in d.items():
                    self.__dict__[k] = v

        options = FakeOptions({'opt1': 'val1', 'opt2': 'val2'})
        required_flags = ['opt1', 'opt2']
        ret = self.common.check_flags(options, required_flags)
        self.assertIsNone(ret)

        options = FakeOptions({'opt1': 'val1', 'opt2': 'val2'})
        required_flags = ['opt1', 'opt2', 'opt3']
        self.assertRaises(exception.Invalid, self.common.check_flags,
                          options, required_flags)

    def test_assert_connector_ok(self):
        self.assertRaises(exception.InvalidInput,
                          self.common._assert_connector_ok, invalid_connector,
                          self.connector_element)
        self.assertIsNone(self.common._assert_connector_ok(
                          connector,
                          self.connector_element))

    @mock.patch.object(STXClient, 'backend_stats')
    def test_update_volume_stats(self, mock_stats):
        mock_stats.side_effect = [stx_exception.RequestError,
                                  stats_large_space]

        self.assertRaises(exception.Invalid, self.common._update_volume_stats)
        mock_stats.assert_called_with(self.common.backend_name,
                                      self.common.backend_type)
        ret = self.common._update_volume_stats()

        self.assertIsNone(ret)
        self.assertEqual({'driver_version': self.common.VERSION,
                          'pools': [{'QoS_support': False,
                                     'multiattach': True,
                                     'free_capacity_gb': 90,
                                     'location_info':
                                     'SeagateVolumeDriver:xxxxx:OpenStack:A',
                                     'pool_name': 'OpenStack',
                                     'total_capacity_gb': 100}],
                          'storage_protocol': None,
                          'vendor_name': 'Seagate',
                          'volume_backend_name': None}, self.common.stats)

    @mock.patch.object(STXClient, 'create_volume')
    def test_create_volume(self, mock_create):
        mock_create.side_effect = [stx_exception.RequestError, None]

        self.assertRaises(exception.Invalid, self.common.create_volume,
                          test_volume)
        ret = self.common.create_volume(test_volume)
        self.assertIsNone(ret)
        mock_create.assert_called_with(encoded_volid,
                                       "%sGiB" % test_volume['size'],
                                       self.common.backend_name,
                                       self.common.backend_type)

    @mock.patch.object(STXClient, 'delete_volume')
    def test_delete_volume(self, mock_delete):
        not_found_e = stx_exception.RequestError(
            'The volume was not found on this system.')
        mock_delete.side_effect = [not_found_e,
                                   stx_exception.RequestError,
                                   None]
        self.assertIsNone(self.common.delete_volume(test_volume))
        self.assertRaises(exception.Invalid, self.common.delete_volume,
                          test_volume)
        self.assertIsNone(self.common.delete_volume(test_volume))
        mock_delete.assert_called_with(encoded_volid)

    @mock.patch.object(STXClient, 'copy_volume')
    @mock.patch.object(STXClient, 'backend_stats')
    def test_create_cloned_volume(self, mock_stats, mock_copy):
        mock_stats.side_effect = [stats_low_space, stats_large_space,
                                  stats_large_space]

        self.assertRaises(
            stx_exception.NotEnoughSpace,
            self.common.create_cloned_volume,
            dest_volume, detached_volume)
        self.assertFalse(mock_copy.called)

        mock_copy.side_effect = [stx_exception.RequestError, None]
        self.assertRaises(exception.Invalid,
                          self.common.create_cloned_volume,
                          dest_volume, detached_volume)

        ret = self.common.create_cloned_volume(dest_volume, detached_volume)
        self.assertIsNone(ret)

        mock_copy.assert_called_with(encoded_volid,
                                     'vqqqqqqqqqqqqqqqqqqq',
                                     self.common.backend_name,
                                     self.common.backend_type)

    @mock.patch.object(STXClient, 'copy_volume')
    @mock.patch.object(STXClient, 'backend_stats')
    @mock.patch.object(STXCommon, 'extend_volume')
    def test_create_cloned_volume_larger(self, mock_extend, mock_stats,
                                         mock_copy):
        mock_stats.side_effect = [stats_low_space, stats_large_space,
                                  stats_large_space]

        self.assertRaises(stx_exception.NotEnoughSpace,
                          self.common.create_cloned_volume,
                          dest_volume_larger, detached_volume)
        self.assertFalse(mock_copy.called)

        mock_copy.side_effect = [stx_exception.RequestError, None]
        self.assertRaises(exception.Invalid,
                          self.common.create_cloned_volume,
                          dest_volume_larger, detached_volume)

        ret = self.common.create_cloned_volume(dest_volume_larger,
                                               detached_volume)
        self.assertIsNone(ret)
        mock_copy.assert_called_with(encoded_volid,
                                     'vqqqqqqqqqqqqqqqqqqq',
                                     self.common.backend_name,
                                     self.common.backend_type)
        mock_extend.assert_called_once_with(dest_volume_larger,
                                            dest_volume_larger['size'])

    @mock.patch.object(STXClient, 'get_volume_size')
    @mock.patch.object(STXClient, 'extend_volume')
    @mock.patch.object(STXClient, 'copy_volume')
    @mock.patch.object(STXClient, 'backend_stats')
    def test_create_volume_from_snapshot(self, mock_stats, mock_copy,
                                         mock_extend, mock_get_size):
        mock_stats.side_effect = [stats_low_space, stats_large_space,
                                  stats_large_space]

        self.assertRaises(stx_exception.NotEnoughSpace,
                          self.common.create_volume_from_snapshot,
                          dest_volume, test_snap)

        mock_copy.side_effect = [stx_exception.RequestError, None]
        mock_get_size.return_value = test_snap['volume_size']
        self.assertRaises(exception.Invalid,
                          self.common.create_volume_from_snapshot,
                          dest_volume, test_snap)

        ret = self.common.create_volume_from_snapshot(dest_volume_larger,
                                                      test_snap)
        self.assertIsNone(ret)
        mock_copy.assert_called_with('sqqqqqqqqqqqqqqqqqqq',
                                     'vqqqqqqqqqqqqqqqqqqq',
                                     self.common.backend_name,
                                     self.common.backend_type)
        mock_extend.assert_called_with('vqqqqqqqqqqqqqqqqqqq', '10GiB')

    @mock.patch.object(STXClient, 'get_volume_size')
    @mock.patch.object(STXClient, 'extend_volume')
    def test_extend_volume(self, mock_extend, mock_size):
        mock_extend.side_effect = [stx_exception.RequestError, None]
        mock_size.side_effect = [10, 10]
        self.assertRaises(exception.Invalid, self.common.extend_volume,
                          test_volume, 20)
        ret = self.common.extend_volume(test_volume, 20)
        self.assertIsNone(ret)
        mock_extend.assert_called_with(encoded_volid, '10GiB')

    @mock.patch.object(STXClient, 'create_snapshot')
    def test_create_snapshot(self, mock_create):
        mock_create.side_effect = [stx_exception.RequestError, None]

        self.assertRaises(exception.Invalid, self.common.create_snapshot,
                          test_snap)
        ret = self.common.create_snapshot(test_snap)
        self.assertIsNone(ret)
        mock_create.assert_called_with(encoded_volid, 'sqqqqqqqqqqqqqqqqqqq')

    @mock.patch.object(STXClient, 'delete_snapshot')
    def test_delete_snapshot(self, mock_delete):
        not_found_e = stx_exception.RequestError(
            'The volume was not found on this system.')
        mock_delete.side_effect = [not_found_e,
                                   stx_exception.RequestError,
                                   None]

        self.assertIsNone(self.common.delete_snapshot(test_snap))
        self.assertRaises(exception.Invalid, self.common.delete_snapshot,
                          test_snap)
        self.assertIsNone(self.common.delete_snapshot(test_snap))
        mock_delete.assert_called_with('sqqqqqqqqqqqqqqqqqqq',
                                       self.common.backend_type)

    @mock.patch.object(STXClient, 'map_volume')
    def test_map_volume(self, mock_map):
        mock_map.side_effect = [stx_exception.RequestError, 10]

        self.assertRaises(exception.Invalid, self.common.map_volume,
                          test_volume, connector, self.connector_element)
        lun = self.common.map_volume(test_volume, connector,
                                     self.connector_element)
        self.assertEqual(10, lun)
        mock_map.assert_called_with(encoded_volid,
                                    connector, self.connector_element)

    @mock.patch.object(STXClient, 'unmap_volume')
    def test_unmap_volume(self, mock_unmap):
        mock_unmap.side_effect = [stx_exception.RequestError, None]

        self.assertRaises(exception.Invalid, self.common.unmap_volume,
                          test_volume, connector, self.connector_element)
        ret = self.common.unmap_volume(test_volume, connector,
                                       self.connector_element)
        self.assertIsNone(ret)
        mock_unmap.assert_called_with(encoded_volid, connector,
                                      self.connector_element)

    @mock.patch.object(STXClient, 'copy_volume')
    @mock.patch.object(STXClient, 'delete_volume')
    @mock.patch.object(STXClient, 'modify_volume_name')
    def test_retype(self, mock_modify, mock_delete, mock_copy):
        mock_copy.side_effect = [stx_exception.RequestError, None]
        self.assertRaises(exception.Invalid, self.common.migrate_volume,
                          test_retype_volume, test_host)
        ret = self.common.migrate_volume(test_retype_volume, test_host)
        self.assertEqual((True, None), ret)
        ret = self.common.migrate_volume(test_retype_volume,
                                         {'capabilities': {}})
        self.assertEqual((False, None), ret)

    @mock.patch.object(STXClient, 'modify_volume_name')
    def test_manage_existing(self, mock_modify):
        existing_ref = {'source-name': 'xxxx'}
        mock_modify.side_effect = [stx_exception.RequestError, None]
        self.assertRaises(exception.Invalid, self.common.manage_existing,
                          test_volume, existing_ref)
        ret = self.common.manage_existing(test_volume, existing_ref)
        self.assertIsNone(ret)

    @mock.patch.object(STXClient, 'get_volume_size')
    def test_manage_existing_get_size(self, mock_volume):
        existing_ref = {'source-name': 'xxxx'}
        mock_volume.side_effect = [stx_exception.RequestError, 1]
        self.assertRaises(exception.Invalid,
                          self.common.manage_existing_get_size,
                          None, existing_ref)
        ret = self.common.manage_existing_get_size(None, existing_ref)
        self.assertEqual(1, ret)

    @mock.patch.object(STXClient, 'modify_volume_name')
    @mock.patch.object(STXClient, '_request')
    def test_manage_existing_snapshot(self, mock_response, mock_modify):
        fake_snap = fake(test_snap)
        mock_response.side_effect = [etree.XML(response_maps),
                                     etree.XML(response_vols)]
        snap_ref = {'source-name': 'snap'}
        ret = self.common.manage_existing_snapshot(fake_snap, snap_ref)
        self.assertIsNone(ret)
        newname = self.common._get_snap_name(test_snap['id'])
        mock_modify.assert_called_with('snap', newname)

    @mock.patch.object(STXClient, 'get_volume_size')
    def test_manage_existing_snapshot_get_size(self, mock_volume):
        existing_ref = {'source-name': 'xxxx'}
        mock_volume.side_effect = [stx_exception.RequestError, 1]
        self.assertRaises(exception.Invalid,
                          self.common.manage_existing_get_size,
                          None, existing_ref)
        ret = self.common.manage_existing_snapshot_get_size(None, existing_ref)
        self.assertEqual(1, ret)

    @mock.patch.object(STXClient, '_request')
    def test_get_manageable_volumes(self, mock_response):
        mock_response.side_effect = [etree.XML(response_maps),
                                     etree.XML(response_vols)]
        cinder_volumes = [fake(id='aa820dc7-851b-4be0-a7a3-7803ab555495')]
        marker = None
        limit = 1000
        offset = 0
        sort_keys = ['size']
        sort_dirs = ['asc']
        ret = self.common.get_manageable_volumes(cinder_volumes,
                                                 marker, limit, offset,
                                                 sort_keys, sort_dirs)
        # We expect to get back 3 volumes: one manageable,
        # one already managed by Cinder, and one mapped (hence unmanageable)
        self.assertEqual(len(ret), 3)
        reasons_not_seen = {'volume in use', 'already managed'}
        manageable_vols = 0
        for vol in ret:
            if vol['reason_not_safe']:
                reasons_not_seen.discard(vol['reason_not_safe'])
            self.assertGreaterEqual(len(vol['reference']['source-name']), 3)
            if vol['safe_to_manage']:
                manageable_vols += 1
                self.assertIsNone(vol.get('cinder-id'))
            else:
                self.assertIsNotNone(vol['reason_not_safe'])

        self.assertEqual(0, len(reasons_not_seen))
        self.assertEqual(1, manageable_vols)

    @mock.patch.object(STXClient, '_request')
    def test_get_manageable_snapshots(self, mock_response):
        mock_response.side_effect = [etree.XML(response_maps),
                                     etree.XML(response_vols)]
        cinder_volumes = [fake(id='aa820dc7-851b-4be0-a7a3-7803ab555495')]
        marker = None
        limit = 1000
        offset = 0
        sort_keys = ['size']
        sort_dirs = ['asc']
        ret = self.common.get_manageable_snapshots(cinder_volumes,
                                                   marker, limit, offset,
                                                   sort_keys, sort_dirs)
        self.assertEqual(ret, [{
            'cinder_id': None,
            'extra_info': None,
            'reason_not_safe': None,
            'reference': {'source-name': 'snap'},
            'safe_to_manage': True,
            'size': 1,
            'source_reference': {'source-name': 'bar'}
        }])


class TestISCSISeagateCommon(TestFCSeagateCommon):
    def setUp(self):
        super(TestISCSISeagateCommon, self).setUp()
        self.connector_element = 'initiator'


class TestSeagateFC(test.TestCase):
    @mock.patch.object(STXCommon, 'do_setup')
    def setUp(self, mock_setup):
        super(TestSeagateFC, self).setUp()
        self.vendor_name = 'Seagate'

        mock_setup.return_value = True

        def fake_init(self, *args, **kwargs):
            super(STXFCDriver, self).__init__()
            self.common = None
            self.configuration = FakeConfiguration1()
            self.lookup_service = fczm_utils.create_lookup_service()

        STXFCDriver.__init__ = fake_init
        self.driver = STXFCDriver()
        self.driver.do_setup(None)

    def _test_with_mock(self, mock, method, args, expected=None):
        func = getattr(self.driver, method)
        mock.side_effect = [exception.Invalid(), None]
        self.assertRaises(exception.Invalid, func, *args)
        self.assertEqual(expected, func(*args))

    @mock.patch.object(STXCommon, 'create_volume')
    def test_create_volume(self, mock_create):
        self._test_with_mock(mock_create, 'create_volume', [None])

    @mock.patch.object(STXCommon,
                       'create_cloned_volume')
    def test_create_cloned_volume(self, mock_create):
        self._test_with_mock(mock_create, 'create_cloned_volume', [None, None])

    @mock.patch.object(STXCommon,
                       'create_volume_from_snapshot')
    def test_create_volume_from_snapshot(self, mock_create):
        self._test_with_mock(mock_create, 'create_volume_from_snapshot',
                             [None, None])

    @mock.patch.object(STXCommon, 'delete_volume')
    def test_delete_volume(self, mock_delete):
        self._test_with_mock(mock_delete, 'delete_volume', [None])

    @mock.patch.object(STXCommon, 'create_snapshot')
    def test_create_snapshot(self, mock_create):
        self._test_with_mock(mock_create, 'create_snapshot', [None])

    @mock.patch.object(STXCommon, 'delete_snapshot')
    def test_delete_snapshot(self, mock_delete):
        self._test_with_mock(mock_delete, 'delete_snapshot', [None])

    @mock.patch.object(STXCommon, 'extend_volume')
    def test_extend_volume(self, mock_extend):
        self._test_with_mock(mock_extend, 'extend_volume', [None, 10])

    @mock.patch.object(STXCommon, 'client_logout')
    @mock.patch.object(STXCommon,
                       'get_active_fc_target_ports')
    @mock.patch.object(STXCommon, 'map_volume')
    @mock.patch.object(STXCommon, 'client_login')
    def test_initialize_connection(self, mock_login, mock_map, mock_ports,
                                   mock_logout):
        mock_login.return_value = None
        mock_logout.return_value = None
        mock_map.side_effect = [exception.Invalid, 1]
        mock_ports.side_effect = [['id1']]

        self.assertRaises(exception.Invalid,
                          self.driver.initialize_connection, test_volume,
                          connector)
        mock_map.assert_called_with(test_volume, connector, 'wwpns')

        ret = self.driver.initialize_connection(test_volume, connector)
        self.assertEqual({'driver_volume_type': 'fibre_channel',
                          'data': {'initiator_target_map': {
                                   '111111111111111': ['id1'],
                                   '111111111111112': ['id1']},
                                   'target_wwn': ['id1'],
                                   'target_lun': 1,
                                   'target_discovered': True}}, ret)

    @mock.patch.object(STXCommon, 'unmap_volume')
    @mock.patch.object(STXClient, 'list_luns_for_host')
    def test_terminate_connection(self, mock_list, mock_unmap):
        mock_unmap.side_effect = [1]
        mock_list.side_effect = ['yes']
        actual = {'driver_volume_type': 'fibre_channel', 'data': {}}
        ret = self.driver.terminate_connection(test_volume, connector)
        self.assertEqual(actual, ret)
        mock_unmap.assert_called_with(test_volume, connector, 'wwpns')
        ret = self.driver.terminate_connection(test_volume, connector)
        self.assertEqual(actual, ret)

    @mock.patch.object(STXCommon, 'get_volume_stats')
    def test_get_volume_stats(self, mock_stats):
        stats = {'storage_protocol': None,
                 'driver_version': self.driver.VERSION,
                 'volume_backend_name': None,
                 'vendor_name': self.vendor_name,
                 'pools': [{'free_capacity_gb': 90,
                            'reserved_percentage': 0,
                            'total_capacity_gb': 100,
                            'QoS_support': False,
                            'multiattach': True,
                            'location_info': 'xx:xx:xx:xx',
                            'pool_name': 'x'}]}
        mock_stats.side_effect = [exception.Invalid, stats, stats]

        self.assertRaises(exception.Invalid, self.driver.get_volume_stats,
                          False)
        ret = self.driver.get_volume_stats(False)
        self.assertEqual(stats, ret)

        ret = self.driver.get_volume_stats(True)
        self.assertEqual(stats, ret)
        mock_stats.assert_called_with(True)

    @mock.patch.object(STXCommon, 'retype')
    def test_retype(self, mock_retype):
        mock_retype.side_effect = [exception.Invalid, True, False]
        args = [None, None, None, None, None]
        self.assertRaises(exception.Invalid, self.driver.retype, *args)
        self.assertTrue(self.driver.retype(*args))
        self.assertFalse(self.driver.retype(*args))

    @mock.patch.object(STXCommon, 'manage_existing')
    def test_manage_existing(self, mock_manage_existing):
        self._test_with_mock(mock_manage_existing, 'manage_existing',
                             [None, None])

    @mock.patch.object(STXCommon,
                       'manage_existing_get_size')
    def test_manage_size(self, mock_manage_size):
        mock_manage_size.side_effect = [exception.Invalid, 1]
        self.assertRaises(exception.Invalid,
                          self.driver.manage_existing_get_size,
                          None, None)
        self.assertEqual(1, self.driver.manage_existing_get_size(None, None))


class TestSeagateISCSI(TestSeagateFC):
    @mock.patch.object(STXCommon, 'do_setup')
    def setUp(self, mock_setup):
        super(TestSeagateISCSI, self).setUp()
        self.vendor_name = 'Seagate'
        mock_setup.return_value = True

        def fake_init(self, *args, **kwargs):
            super(STXISCSIDriver, self).__init__()
            self.common = None
            self.configuration = FakeConfiguration2()
            self.iscsi_ips = ['10.0.0.11']

        STXISCSIDriver.__init__ = fake_init
        self.driver = STXISCSIDriver()
        self.driver.do_setup(None)

    @mock.patch.object(STXCommon, 'client_logout')
    @mock.patch.object(STXCommon,
                       'get_active_iscsi_target_portals')
    @mock.patch.object(STXCommon,
                       'get_active_iscsi_target_iqns')
    @mock.patch.object(STXCommon, 'map_volume')
    @mock.patch.object(STXCommon, 'client_login')
    def test_initialize_connection(self, mock_login, mock_map, mock_iqns,
                                   mock_portals, mock_logout):
        mock_login.return_value = None
        mock_logout.return_value = None
        mock_map.side_effect = [exception.Invalid, 1]
        self.driver.iscsi_ips = ['10.0.0.11']
        self.driver.initialize_iscsi_ports()
        mock_iqns.side_effect = [['id2']]
        mock_portals.return_value = {'10.0.0.11': 'Up', '10.0.0.12': 'Up'}

        self.assertRaises(exception.Invalid,
                          self.driver.initialize_connection, test_volume,
                          connector)
        mock_map.assert_called_with(test_volume, connector, 'initiator')

        ret = self.driver.initialize_connection(test_volume, connector)
        self.assertEqual({'driver_volume_type': 'iscsi',
                          'data': {'target_iqn': 'id2',
                                   'target_lun': 1,
                                   'target_discovered': True,
                                   'target_portal': '10.0.0.11:3260'}}, ret)

    @mock.patch.object(STXCommon, 'unmap_volume')
    def test_terminate_connection(self, mock_unmap):
        mock_unmap.side_effect = [exception.Invalid, 1]

        self.assertRaises(exception.Invalid,
                          self.driver.terminate_connection, test_volume,
                          connector)
        mock_unmap.assert_called_with(test_volume, connector, 'initiator')

        ret = self.driver.terminate_connection(test_volume, connector)
        self.assertIsNone(ret)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sqlalchemy as sa

from cinder import exception
from cinder.i18n import _


def upgrade(migrate_engine):
    """Make volume_type columns non-nullable"""

    meta = sa.MetaData(bind=migrate_engine)

    # Update volume_type columns in tables to not allow null value

    volumes = sa.Table('volumes', meta, autoload=True)

    try:
        volumes.c.volume_type_id.alter(nullable=False)
    except Exception:
        msg = (_('Migration cannot continue until all volumes have '
                 'been migrated to the `__DEFAULT__` volume type. Please '
                 'run `cinder-manage db online_data_migrations`. '
                 'There are still untyped volumes unmigrated.'))
        raise exception.ValidationError(msg)

    snapshots = sa.Table('snapshots', meta, autoload=True)

    try:
        snapshots.c.volume_type_id.alter(nullable=False)
    except Exception:
        msg = (_('Migration cannot continue until all snapshots have '
                 'been migrated to the `__DEFAULT__` volume type. Please '
                 'run `cinder-manage db online_data_migrations`.'
                 'There are still %(count)i untyped snapshots unmigrated.'))
        raise exception.ValidationError(msg)

    encryption = sa.Table('encryption', meta, autoload=True)
    # since volume_type is a mandatory arg when creating encryption
    # volume_type_id column won't contain any null values so we can directly
    # alter it
    encryption.c.volume_type_id.alter(nullable=False)



#    (c) Copyright 2013 Brocade Communications Systems Inc.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""Unit tests for fc san lookup service."""

from cinder import exception
from cinder.tests.unit import test
from cinder.volume import configuration as conf
from cinder.zonemanager import fc_san_lookup_service as san_service

_target_ns_map = {'100000051e55a100': ['20240002ac000a50']}
_initiator_ns_map = {'100000051e55a100': ['10008c7cff523b01']}
_device_map_to_verify = {
    '100000051e55a100': {
        'initiator_port_wwn_list': [
            '10008c7cff523b01'], 'target_port_wwn_list': ['20240002ac000a50']}}
_fabric_wwn = '100000051e55a100'


class TestFCSanLookupService(san_service.FCSanLookupService, test.TestCase):

    def setUp(self):
        super(TestFCSanLookupService, self).setUp()
        self.configuration = self.setup_config()

    # override some of the functions
    def __init__(self, *args, **kwargs):
        test.TestCase.__init__(self, *args, **kwargs)

    def setup_config(self):
        configuration = conf.Configuration(None)
        # fill up config
        configuration.fc_san_lookup_service = (
            'cinder.tests.unit.zonemanager.test_brcd_lookup_service.'
            'FakeBrcdFCSanLookupService')
        return configuration

    def test_get_device_mapping_from_network(self):
        GlobalParams._is_normal_test = True
        initiator_list = ['10008c7cff523b01']
        target_list = ['20240002ac000a50', '20240002ac000a40']
        device_map = self.get_device_mapping_from_network(
            initiator_list, target_list)
        self.assertDictEqual(_device_map_to_verify, device_map)

    def test_get_device_mapping_from_network_for_invalid_config(self):
        GlobalParams._is_normal_test = False
        initiator_list = ['10008c7cff523b01']
        target_list = ['20240002ac000a50', '20240002ac000a40']
        self.assertRaises(exception.FCSanLookupServiceException,
                          self.get_device_mapping_from_network,
                          initiator_list, target_list)


class FakeBrcdFCSanLookupService(object):

    def __init__(self, **kwargs):
        pass

    def get_device_mapping_from_network(self,
                                        initiator_wwn_list,
                                        target_wwn_list):
        if not GlobalParams._is_normal_test:
            raise exception.FCSanLookupServiceException("Error")
        device_map = {}
        initiators = []
        targets = []
        for i in initiator_wwn_list:
            if (i in _initiator_ns_map[_fabric_wwn]):
                initiators.append(i)
        for t in target_wwn_list:
            if (t in _target_ns_map[_fabric_wwn]):
                targets.append(t)
        device_map[_fabric_wwn] = {
            'initiator_port_wwn_list': initiators,
            'target_port_wwn_list': targets}
        return device_map


class GlobalParams(object):
    global _is_normal_test
    _is_normal_test = True



# Copyright (c) 2016 by Kaminario Technologies, Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Unit tests for kaminario driver."""
import re
import time
from unittest import mock

import ddt
from oslo_config import cfg
from oslo_utils import units

from cinder import context
from cinder import exception
from cinder import objects
from cinder.objects import fields
from cinder.tests.unit import fake_snapshot
from cinder.tests.unit import fake_volume
from cinder.tests.unit import test
from cinder.volume import configuration
from cinder.volume.drivers.kaminario import kaminario_common
from cinder.volume.drivers.kaminario import kaminario_fc
from cinder.volume.drivers.kaminario import kaminario_iscsi
from cinder.volume import volume_utils

CONF = cfg.CONF

CONNECTOR = {'initiator': 'iqn.1993-08.org.debian:01:12aa12aa12aa',
             'ip': '192.168.2.5', 'platform': 'x86_64', 'host': 'test-k2',
             'wwpns': ['12341a2a00001234', '12341a2a00001235'],
             'wwnns': ['12351a2a00001234', '12361a2a00001234'],
             'os_type': 'linux2', 'multipath': False}


class FakeK2Obj(object):
    id = 548
    lun = 548


class FakeSaveObject(FakeK2Obj):
    def __init__(self, *args, **kwargs):
        item = kwargs.pop('item', 1)
        self.ntype = kwargs.get('ntype')
        self.ip_address = '10.0.0.%s' % item
        self.iscsi_qualified_target_name = "xyztlnxyz"
        self.snapshot = FakeK2Obj()
        self.name = 'test'
        self.pwwn = '50024f405330030%s' % item
        self.volume_group = self
        self.is_dedup = True
        self.size = units.Mi
        self.replication_status = None
        self.state = 'in_sync'
        self.generation_number = 548
        self.current_role = 'target'
        self.current_snapshot_progress = 100
        self.current_snapshot_id = None
        self.wan_port = None

    def refresh(self):
        return

    def save(self):
        return FakeSaveObject()

    def delete(self):
        return None


class FakeSaveObjectExp(FakeSaveObject):
    def save(self):
        raise kaminario_common.KaminarioCinderDriverException("test")

    def delete(self):
        raise kaminario_common.KaminarioCinderDriverException("test")


class FakeSearchObject(object):
    hits = [FakeSaveObject(item=1), FakeSaveObject(item=2)]
    total = 2

    def __init__(self, *args):
        if args and "mappings" in args[0]:
            self.total = 0


class FakeSearchObjectExp(object):
    hits = [FakeSaveObjectExp()]
    total = 1


class FakeKrest(object):
    def search(self, *args, **argv):
        return FakeSearchObject(*args)

    def new(self, *args, **argv):
        return FakeSaveObject()


class FakeKrestException(object):
    def search(self, *args, **argv):
        return FakeSearchObjectExp()

    def new(self, *args, **argv):
        return FakeSaveObjectExp()


class Replication(object):
    backend_id = '10.0.0.1'
    login = 'login'
    password = 'password'
    rpo = 500


@ddt.ddt
class TestKaminarioCommon(test.TestCase):
    driver = None
    conf = None

    def setUp(self):
        self._setup_config()
        self._setup_driver()
        super(TestKaminarioCommon, self).setUp()
        self.context = context.get_admin_context()
        self.vol = fake_volume.fake_volume_obj(self.context)
        self.vol.volume_type = fake_volume.fake_volume_type_obj(self.context)
        self.vol.volume_type.extra_specs = {'foo': None}
        self.snap = fake_snapshot.fake_snapshot_obj(self.context)
        self.snap.volume = self.vol
        self.patch('eventlet.sleep')

    def _setup_config(self):
        self.conf = mock.Mock(spec=configuration.Configuration)
        self.conf.kaminario_dedup_type_name = "dedup"
        self.conf.volume_dd_blocksize = 2
        self.conf.disable_discovery = False
        self.conf.unique_fqdn_network = True

    def _setup_driver(self):
        self.driver = (kaminario_iscsi.
                       KaminarioISCSIDriver(configuration=self.conf))
        device = mock.Mock(return_value={'device': {'path': '/dev'}})
        self.driver._connect_device = device
        self.driver.client = FakeKrest()

    def test_create_volume(self):
        """Test create_volume."""
        result = self.driver.create_volume(self.vol)
        self.assertIsNone(result)

    def test_create_volume_with_exception(self):
        """Test create_volume_with_exception."""
        self.driver.client = FakeKrestException()
        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                          self.driver.create_volume, self.vol)

    def test_delete_volume(self):
        """Test delete_volume."""
        result = self.driver.delete_volume(self.vol)
        self.assertIsNone(result)

    def test_delete_volume_with_exception(self):
        """Test delete_volume_with_exception."""
        self.driver.client = FakeKrestException()
        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                          self.driver.delete_volume, self.vol)

    def test_create_snapshot(self):
        """Test create_snapshot."""
        self.snap.id = "253b2878-ec60-4793-ad19-e65496ec7aab"
        self.driver.client.new = mock.Mock()
        result = self.driver.create_snapshot(self.snap)
        self.assertIsNone(result)
        fake_object = self.driver.client.search().hits[0]
        self.driver.client.new.assert_called_once_with(
            "snapshots",
            short_name='cs-253b2878-ec60-4793-ad19-e65496ec7aab',
            source=fake_object, retention_policy=fake_object,
            is_auto_deleteable=False)

    def test_create_snapshot_with_exception(self):
        """Test create_snapshot_with_exception."""
        self.driver.client = FakeKrestException()
        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                          self.driver.create_snapshot, self.snap)

    def test_delete_snapshot(self):
        """Test delete_snapshot."""
        result = self.driver.delete_snapshot(self.snap)
        self.assertIsNone(result)

    def test_delete_snapshot_with_exception(self):
        """Test delete_snapshot_with_exception."""
        self.driver.client = FakeKrestException()
        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                          self.driver.delete_snapshot, self.snap)

    @mock.patch.object(volume_utils, 'brick_get_connector_properties')
    @mock.patch.object(volume_utils, 'copy_volume')
    def test_create_volume_from_snapshot(self, mock_copy_volume,
                                         mock_brick_get):
        """Test create_volume_from_snapshot."""
        mock_brick_get.return_value = CONNECTOR
        mock_copy_volume.return_value = None
        self.driver._kaminario_disconnect_volume = mock.Mock()
        result = self.driver.create_volume_from_snapshot(self.vol, self.snap)
        self.assertIsNone(result)

    @mock.patch.object(volume_utils, 'brick_get_connector_properties')
    @mock.patch.object(volume_utils, 'copy_volume')
    def test_create_volume_from_snapshot_with_exception(self, mock_copy_volume,
                                                        mock_brick_get):
        """Test create_volume_from_snapshot_with_exception."""
        mock_brick_get.return_value = CONNECTOR
        mock_copy_volume.return_value = None
        self.driver.client = FakeKrestException()
        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                          self.driver.create_volume_from_snapshot, self.vol,
                          self.snap)

    @mock.patch.object(volume_utils, 'brick_get_connector_properties')
    @mock.patch.object(volume_utils, 'copy_volume')
    def test_create_cloned_volume(self, mock_copy_volume, mock_brick_get):
        """Test create_cloned_volume."""
        mock_brick_get.return_value = CONNECTOR
        mock_copy_volume.return_value = None
        self.driver._kaminario_disconnect_volume = mock.Mock()
        result = self.driver.create_cloned_volume(self.vol, self.vol)
        self.assertIsNone(result)

    @mock.patch.object(volume_utils, 'brick_get_connector_properties')
    @mock.patch.object(volume_utils, 'copy_volume')
    def test_create_cloned_volume_with_exception(self, mock_copy_volume,
                                                 mock_brick_get):
        """Test create_cloned_volume_with_exception."""
        mock_brick_get.return_value = CONNECTOR
        mock_copy_volume.return_value = None
        self.driver.terminate_connection = mock.Mock()
        self.driver.client = FakeKrestException()
        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                          self.driver.create_cloned_volume, self.vol, self.vol)

    def test_extend_volume(self):
        """Test extend_volume."""
        new_size = 256
        result = self.driver.extend_volume(self.vol, new_size)
        self.assertIsNone(result)

    def test_extend_volume_with_exception(self):
        """Test extend_volume_with_exception."""
        self.driver.client = FakeKrestException()
        new_size = 256
        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                          self.driver.extend_volume, self.vol, new_size)

    def test_initialize_connection_with_exception(self):
        """Test initialize_connection_with_exception."""
        self.driver.client = FakeKrestException()
        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                          self.driver.initialize_connection, self.vol,
                          CONNECTOR)

    def test_get_lun_number(self):
        """Test _get_lun_number."""
        host, host_rs, host_name = self.driver._get_host_object(CONNECTOR)
        result = self.driver._get_lun_number(self.vol, host)
        self.assertEqual(548, result)

    def test_get_volume_object(self):
        """Test _get_volume_object."""
        result = self.driver._get_volume_object(self.vol)
        self.assertEqual(548, result.id)

    def test_get_host_object(self):
        """Test _get_host_object."""
        host, host_rs, host_name = self.driver._get_host_object(CONNECTOR)
        self.assertEqual(548, host.id)
        self.assertEqual(2, host_rs.total)
        self.assertEqual('test-k2', host_name)

    def test_k2_initialize_connection(self):
        """Test k2_initialize_connection."""
        result = self.driver.k2_initialize_connection(self.vol, CONNECTOR)
        self.assertEqual(548, result)

    @mock.patch.object(FakeSearchObject, 'total', 1)
    def test_manage_existing(self):
        """Test manage_existing."""
        self.driver._get_replica_status = mock.Mock(return_value=False)
        result = self.driver.manage_existing(self.vol, {'source-name': 'test'})
        self.assertIsNone(result)

    def test_manage_existing_exp(self):
        self.driver._get_replica_status = mock.Mock(return_value=True)
        self.assertRaises(exception.ManageExistingInvalidReference,
                          self.driver.manage_existing, self.vol,
                          {'source-name': 'test'})

    def test_manage_vg_volumes(self):
        self.driver.nvol = 2
        self.assertRaises(exception.ManageExistingInvalidReference,
                          self.driver.manage_existing, self.vol,
                          {'source-name': 'test'})

    def test_manage_existing_get_size(self):
        """Test manage_existing_get_size."""
        self.driver.client.search().hits[0].size = units.Mi
        result = self.driver.manage_existing_get_size(self.vol,
                                                      {'source-name': 'test'})
        self.assertEqual(1, result)

    def test_get_is_dedup(self):
        """Test _get_is_dedup."""
        result = self.driver._get_is_dedup(self.vol.volume_type)
        self.assertTrue(result)

    def test_get_is_dedup_false(self):
        """Test _get_is_dedup_false."""
        specs = {'kaminario:thin_prov_type': 'nodedup'}
        self.vol.volume_type.extra_specs = specs
        result = self.driver._get_is_dedup(self.vol.volume_type)
        self.assertFalse(result)

    def test_get_replica_status(self):
        """Test _get_replica_status."""
        result = self.driver._get_replica_status(self.vol)
        self.assertTrue(result)

    def test_create_volume_replica(self):
        """Test _create_volume_replica."""
        vg = FakeSaveObject()
        rep = Replication()
        self.driver.replica = rep
        session_name = self.driver.get_session_name('1234567890987654321')
        self.assertEqual('ssn-1234567890987654321', session_name)
        rsession_name = self.driver.get_rep_name(session_name)
        self.assertEqual('rssn-1234567890987654321', rsession_name)
        src_ssn = self.driver.client.new("replication/sessions").save()
        self.assertEqual('in_sync', src_ssn.state)
        result = self.driver._create_volume_replica(self.vol, vg, vg, rep.rpo)
        self.assertIsNone(result)

    def test_create_volume_replica_exp(self):
        """Test _create_volume_replica_exp."""
        vg = FakeSaveObject()
        rep = Replication()
        self.driver.replica = rep
        self.driver.client = FakeKrestException()
        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                          self.driver._create_volume_replica, self.vol,
                          vg, vg, rep.rpo)

    def test_delete_by_ref(self):
        """Test _delete_by_ref."""
        result = self.driver._delete_by_ref(self.driver.client, 'volume',
                                            'name', 'message')
        self.assertIsNone(result)

    def test_failover_volume(self):
        """Test _failover_volume."""
        self.driver.target = FakeKrest()
        session_name = self.driver.get_session_name('1234567890987654321')
        self.assertEqual('ssn-1234567890987654321', session_name)
        rsession_name = self.driver.get_rep_name(session_name)
        self.assertEqual('rssn-1234567890987654321', rsession_name)
        result = self.driver._failover_volume(self.vol)
        self.assertIsNone(result)

    @mock.patch.object(kaminario_common.KaminarioCinderDriver,
                       '_check_for_status')
    @mock.patch.object(objects.service.Service, 'get_by_args')
    def test_failover_host(self, get_by_args, check_stauts):
        """Test failover_host."""
        mock_args = mock.Mock()
        mock_args.active_backend_id = '10.0.0.1'
        self.vol.replication_status = 'failed-over'
        self.driver.configuration.san_ip = '10.0.0.1'
        get_by_args.side_effect = [mock_args, mock_args]
        self.driver.host = 'host'
        volumes = [self.vol, self.vol]
        self.driver.replica = Replication()
        self.driver.target = FakeKrest()
        self.driver.target.search().total = 1
        self.driver.client.search().total = 1
        backend_ip, res_volumes, __ = self.driver.failover_host(
            None, volumes, [])
        self.assertEqual('10.0.0.1', backend_ip)
        status = res_volumes[0]['updates']['replication_status']
        self.assertEqual(fields.ReplicationStatus.FAILED_OVER, status)
        # different backend ip
        self.driver.configuration.san_ip = '10.0.0.2'
        self.driver.client.search().hits[0].state = 'in_sync'
        backend_ip, res_volumes, __ = self.driver.failover_host(
            None, volumes, [])
        self.assertEqual('10.0.0.2', backend_ip)
        status = res_volumes[0]['updates']['replication_status']
        self.assertEqual(fields.ReplicationStatus.DISABLED, status)

    def test_delete_volume_replica(self):
        """Test _delete_volume_replica."""
        self.driver.replica = Replication()
        self.driver.target = FakeKrest()
        session_name = self.driver.get_session_name('1234567890987654321')
        self.assertEqual('ssn-1234567890987654321', session_name)
        rsession_name = self.driver.get_rep_name(session_name)
        self.assertEqual('rssn-1234567890987654321', rsession_name)
        res = self.driver._delete_by_ref(self.driver.client, 'volumes',
                                         'test', 'test')
        self.assertIsNone(res)
        result = self.driver._delete_volume_replica(self.vol, 'test', 'test')
        self.assertIsNone(result)
        src_ssn = self.driver.client.search("replication/sessions").hits[0]
        self.assertEqual('idle', src_ssn.state)

    def test_delete_volume_replica_exp(self):
        """Test _delete_volume_replica_exp."""
        self.driver.replica = Replication()
        self.driver.target = FakeKrestException()
        self.driver._check_for_status = mock.Mock()
        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                          self.driver._delete_volume_replica, self.vol,
                          'test', 'test')

    def test_get_is_replica(self):
        """Test get_is_replica."""
        result = self.driver._get_is_replica(self.vol.volume_type)
        self.assertFalse(result)

    def test_get_is_replica_true(self):
        """Test get_is_replica_true."""
        self.driver.replica = Replication()
        self.vol.volume_type.extra_specs = {'kaminario:replication': 'enabled'}
        result = self.driver._get_is_replica(self.vol.volume_type)
        self.assertTrue(result)

    def test_after_volume_copy(self):
        """Test after_volume_copy."""
        result = self.driver.after_volume_copy(None, self.vol,
                                               self.vol.volume_type)
        self.assertIsNone(result)

    def test_retype(self):
        """Test retype."""
        replica_status = self.driver._get_replica_status('test')
        self.assertTrue(replica_status)
        replica = self.driver._get_is_replica(self.vol.volume_type)
        self.assertFalse(replica)
        self.driver.replica = Replication()
        result = self.driver._add_replication(self.vol)
        self.assertIsNone(result)
        self.driver.target = FakeKrest()
        self.driver._check_for_status = mock.Mock()
        result = self.driver._delete_replication(self.vol)
        self.assertIsNone(result)
        self.driver._delete_volume_replica = mock.Mock()
        result = self.driver.retype(None, self.vol,
                                    self.vol.volume_type, None, None)
        self.assertTrue(result)
        new_vol_type = fake_volume.fake_volume_type_obj(self.context)
        new_vol_type.extra_specs = {'kaminario:thin_prov_type': 'nodedup'}
        result2 = self.driver.retype(None, self.vol,
                                     new_vol_type, None, None)
        self.assertFalse(result2)

    def test_add_replication(self):
        """"Test _add_replication."""
        self.driver.replica = Replication()
        result = self.driver._add_replication(self.vol)
        self.assertIsNone(result)

    def test_delete_replication(self):
        """Test _delete_replication."""
        self.driver.replica = Replication()
        self.driver.target = FakeKrest()
        self.driver._check_for_status = mock.Mock()
        result = self.driver._delete_replication(self.vol)
        self.assertIsNone(result)

    def test_create_failover_volume_replica(self):
        """Test _create_failover_volume_replica."""
        self.driver.replica = Replication()
        self.driver.target = FakeKrest()
        self.driver.configuration.san_ip = '10.0.0.1'
        result = self.driver._create_failover_volume_replica(self.vol,
                                                             'test', 'test')
        self.assertIsNone(result)

    def test_create_volume_replica_user_snap(self):
        """Test create_volume_replica_user_snap."""
        result = self.driver._create_volume_replica_user_snap(FakeKrest(),
                                                              'sess')
        self.assertEqual(548, result)

    def test_is_user_snap_sync_finished(self):
        """Test _is_user_snap_sync_finished."""
        sess_mock = mock.Mock()
        sess_mock.refresh = mock.Mock()
        sess_mock.generation_number = 548
        sess_mock.current_snapshot_id = None
        sess_mock.current_snapshot_progress = 100
        sess_mock.current_snapshot_id = None
        self.driver.snap_updates = [{'tgt_ssn': sess_mock, 'gno': 548,
                                     'stime': time.time()}]
        result = self.driver._is_user_snap_sync_finished()
        self.assertIsNone(result)

    def test_delete_failover_volume_replica(self):
        """Test _delete_failover_volume_replica."""
        self.driver.target = FakeKrest()
        result = self.driver._delete_failover_volume_replica(self.vol, 'test',
                                                             'test')
        self.assertIsNone(result)

    def test_get_initiator_host_name(self):
        result = self.driver.get_initiator_host_name(CONNECTOR)
        self.assertEqual(CONNECTOR['host'], result)

    @ddt.data(True, False)
    def test_get_initiator_host_name_unique(self, in_shared):
        cfg = self._set_unique_fqdn_override(False, in_shared)
        self.mock_object(self.driver, 'configuration', cfg)
        result = self.driver.get_initiator_host_name(CONNECTOR)
        expected = re.sub('[:.]', '_', CONNECTOR['initiator'][::-1][:32])
        self.assertEqual(expected, result)


@ddt.ddt
class TestKaminarioISCSI(TestKaminarioCommon):
    def test_get_target_info(self):
        """Test get_target_info."""
        iscsi_portals, target_iqns = self.driver.get_target_info(self.vol)
        self.assertEqual(['10.0.0.1:3260', '10.0.0.2:3260'],
                         iscsi_portals)
        self.assertEqual(['xyztlnxyz', 'xyztlnxyz'],
                         target_iqns)

    @ddt.data(True, False)
    def test_initialize_connection(self, multipath):
        """Test initialize_connection."""
        connector = CONNECTOR.copy()
        connector['multipath'] = multipath
        self.driver.configuration.disable_discovery = False

        conn_info = self.driver.initialize_connection(self.vol, CONNECTOR)
        expected = {
            'data': {
                'target_discovered': True,
                'target_iqn': 'xyztlnxyz',
                'target_lun': 548,
                'target_portal': '10.0.0.1:3260',
            },
            'driver_volume_type': 'iscsi',
        }
        self.assertEqual(expected, conn_info)

    def test_initialize_connection_multipath(self):
        """Test initialize_connection with multipath."""
        connector = CONNECTOR.copy()
        connector['multipath'] = True
        self.driver.configuration.disable_discovery = True

        conn_info = self.driver.initialize_connection(self.vol, connector)

        expected = {
            'data': {
                'target_discovered': True,
                'target_iqn': 'xyztlnxyz',
                'target_iqns': ['xyztlnxyz', 'xyztlnxyz'],
                'target_lun': 548,
                'target_luns': [548, 548],
                'target_portal': '10.0.0.1:3260',
                'target_portals': ['10.0.0.1:3260', '10.0.0.2:3260'],
            },
            'driver_volume_type': 'iscsi',
        }
        self.assertEqual(expected, conn_info)

    def test_terminate_connection(self):
        """Test terminate_connection."""
        result = self.driver.terminate_connection(self.vol, CONNECTOR)
        self.assertIsNone(result)

    def test_terminate_connection_without_connector(self):
        """Test terminate_connection_without_connector."""
        result = self.driver.terminate_connection(self.vol, None)
        self.assertIsNone(result)


@ddt.ddt
class TestKaminarioFC(TestKaminarioCommon):

    def _setup_driver(self):
        self.driver = (kaminario_fc.
                       KaminarioFCDriver(configuration=self.conf))
        device = mock.Mock(return_value={'device': {'path': '/dev'}})
        self.driver._connect_device = device
        self.driver.client = FakeKrest()
        self.driver._lookup_service = mock.Mock()

    def test_initialize_connection(self):
        """Test initialize_connection."""
        conn_info = self.driver.initialize_connection(self.vol, CONNECTOR)
        self.assertIn('data', conn_info)
        self.assertIn('target_wwn', conn_info['data'])

    def test_get_target_info(self):
        """Test get_target_info."""
        target_wwpn = self.driver.get_target_info(self.vol)
        self.assertEqual(['50024f4053300301', '50024f4053300302'],
                         target_wwpn)

    def test_terminate_connection(self):
        """Test terminate_connection."""
        result = self.driver.terminate_connection(self.vol, CONNECTOR)
        self.assertIn('data', result)

    def test_terminate_connection_without_connector(self):
        """Test terminate_connection_without_connector."""
        result = self.driver.terminate_connection(self.vol, None)
        self.assertIn('data', result)

    @ddt.data(True, False)
    def test_get_initiator_host_name_unique(self, in_shared):
        cfg = self._set_unique_fqdn_override(False, in_shared)
        self.mock_object(self.driver, 'configuration', cfg)
        connector = CONNECTOR.copy()
        del connector['initiator']
        result = self.driver.get_initiator_host_name(connector)
        expected = re.sub('[:.]', '_', connector['wwnns'][0][::-1][:32])
        self.assertEqual(expected, result)
